package com.github.pratikalladi;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import twitter4j.Status;
import twitter4j.TwitterException;
import twitter4j.TwitterObjectFactory;

import java.text.ParseException;
import java.util.ArrayList;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.spy;

@RunWith(MockitoJUnitRunner.class)
public class PoligraphTest {

    private Poligraph poligraph;

    @Mock TwitterSentimentAnalysis twitterSentimentAnalysis;

    @Before
    public void setUp() throws Exception {
        poligraph = spy(new Poligraph());
    }

    @Test
    public void testGetUserTweetsInRange() throws TwitterException, ParseException {
        ArrayList<Status> statuses = new ArrayList<>();
        String jsonTweet1 =
                "{\"created_at\":\"Wed Nov 15 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"RT @80Level: Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and…\",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";
        String jsonTweet2 =
                "{\"created_at\":\"Wed Nov 18 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"id_str\":\"1329079661272342529\",\"full_text\":\"RT @80Level: Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and…\",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";

        statuses.add(TwitterObjectFactory.createStatus(jsonTweet1));
        statuses.add(TwitterObjectFactory.createStatus(jsonTweet2));

        doReturn(statuses).when(poligraph).getUserTweets(any(String.class));
        ArrayList<Status> result = poligraph.getUserTweetsInRange("user", "2020-11-17", "2020-11-19");
        assertThat(result.size(), is(1));
    }

    @Test
    public void testGetUserTweetsContainsWord() throws TwitterException, ParseException {
        ArrayList<Status> statuses = new ArrayList<>();
        String jsonTweet1 =
                "{\"created_at\":\"Wed Nov 15 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"RT @80Level: Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and…\",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";
        String jsonTweet2 =
                "{\"created_at\":\"Wed Nov 18 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"@TikTok cool app \",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";

        statuses.add(TwitterObjectFactory.createStatus(jsonTweet1));
        statuses.add(TwitterObjectFactory.createStatus(jsonTweet2));

        doReturn(statuses).when(poligraph).getUserTweets(any(String.class));
        ArrayList<Status> result = poligraph.getUserTweetsContainsWord("user", "iPhone");
        assertThat(result.size(), is(1));
    }

    @Test
    public void testGetUserTweetsContainsStringInRange() throws TwitterException, ParseException {
        ArrayList<Status> statuses = new ArrayList<>();
        String jsonTweet1 =
                "{\"created_at\":\"Wed Nov 15 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"RT @80Level: Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and…\",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";
        String jsonTweet2 =
                "{\"created_at\":\"Wed Nov 18 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"awesome @TikTok cool app \",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";

        statuses.add(TwitterObjectFactory.createStatus(jsonTweet1));
        statuses.add(TwitterObjectFactory.createStatus(jsonTweet2));

        doReturn(statuses).when(poligraph).getUserTweets(any(String.class));
        ArrayList<Status> result =
                poligraph.getUserTweetsContainsStringInRange("user", "awesome", "2020-11-17", "2020-11-19");
        assertThat(result.size(), is(1));
    }

    @Test
    public void testAnalyzeSentiment() throws TwitterException, ParseException {
        String jsonTweet1 =
                "{\"created_at\":\"Wed Nov 15 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"RT @80Level: Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and…\",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";

        doReturn(twitterSentimentAnalysis).when(poligraph).getTwitterSentimentAnalysis();
        doReturn(1).when(twitterSentimentAnalysis).analyzeTweet(any(String.class));
        Result result = poligraph.analyzeSentiment(TwitterObjectFactory.createStatus(jsonTweet1));
        assertNotNull(result);
        assertThat(result.getSentimentValue(), is(1));
        assertThat(result.getSentimentString(), is("Negative"));
        assertThat(result.getStatus(), is(TwitterObjectFactory.createStatus(jsonTweet1)));
    }

    @Test
    public void testAnalyzeSentiments() throws TwitterException, ParseException {
        ArrayList<Status> statuses = new ArrayList<>();
        String jsonTweet1 =
                "{\"created_at\":\"Wed Nov 15 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"RT @80Level: Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and…\",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";
        String jsonTweet2 =
                "{\"created_at\":\"Wed Nov 18 15:10:59 +0000 2020\",\"id\":1329079661272342529,\"full_text\":\"awesome @TikTok cool app \",\"truncated\":false,\"display_text_range\":[0,140],\"entities\":{\"hashtags\":[],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"80Level\",\"name\":\"80 LEVEL\",\"id\":2913699628,\"id_str\":\"2913699628\",\"indices\":[3,11]},{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[48,57]}],\"urls\":[]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":588458496,\"id_str\":\"588458496\",\"name\":\"\uD83C\uDF0A\",\"screen_name\":\"vague_bleue_\",\"location\":\"\",\"description\":\"\",\"url\":null,\"entities\":{\"description\":{\"urls\":[]}},\"protected\":false,\"followers_count\":24,\"friends_count\":1154,\"listed_count\":1,\"created_at\":\"Wed May 23 15:42:39 +0000 2012\",\"favourites_count\":3544,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":false,\"statuses_count\":2047,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"C0DEED\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/2243800944\\/Photoshop-Filtre-esthetique-extrusion_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/588458496\\/1588458579\",\"profile_link_color\":\"1DA1F2\",\"profile_sidebar_border_color\":\"C0DEED\",\"profile_sidebar_fill_color\":\"DDEEF6\",\"profile_text_color\":\"333333\",\"profile_use_background_image\":true,\"has_extended_profile\":false,\"default_profile\":true,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"retweeted_status\":{\"created_at\":\"Wed Nov 18 14:28:26 +0000 2020\",\"id\":1329068955009368066,\"id_str\":\"1329068955009368066\",\"full_text\":\"Check out a cool new voxel art app @VoxelMax for iPhone and iPad with a suite of sculpting tools, a flexible scene editor, and an asset manager:\\n\\nhttps:\\/\\/t.co\\/xFjJXJQp3w\\n\\n#voxelart #voxels #3dart #iphone #ipad #appstore #3dmodeling #apple \\n\\nScene created by Thibault Simar https:\\/\\/t.co\\/rvyVPAJyxU\",\"truncated\":false,\"display_text_range\":[0,272],\"entities\":{\"hashtags\":[{\"text\":\"voxelart\",\"indices\":[171,180]},{\"text\":\"voxels\",\"indices\":[181,188]},{\"text\":\"3dart\",\"indices\":[189,195]},{\"text\":\"iphone\",\"indices\":[196,203]},{\"text\":\"ipad\",\"indices\":[204,209]},{\"text\":\"appstore\",\"indices\":[210,219]},{\"text\":\"3dmodeling\",\"indices\":[220,231]},{\"text\":\"apple\",\"indices\":[232,238]}],\"symbols\":[],\"user_mentions\":[{\"screen_name\":\"VoxelMax\",\"name\":\"Voxel Max\",\"id\":1104051406993002506,\"id_str\":\"1104051406993002506\",\"indices\":[35,44]}],\"urls\":[{\"url\":\"https:\\/\\/t.co\\/xFjJXJQp3w\",\"expanded_url\":\"https:\\/\\/80.lv\\/articles\\/voxel-art-app-for-iphone-and-ipad\\/\",\"display_url\":\"80.lv\\/articles\\/voxel…\",\"indices\":[146,169]}],\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"photo\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}}}]},\"extended_entities\":{\"media\":[{\"id\":1329049728609378304,\"id_str\":\"1329049728609378304\",\"indices\":[273,296],\"media_url\":\"http:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"media_url_https\":\"https:\\/\\/pbs.twimg.com\\/ext_tw_video_thumb\\/1329049728609378304\\/pu\\/img\\/MFxS4acjZ8RsB7UI.jpg\",\"url\":\"https:\\/\\/t.co\\/rvyVPAJyxU\",\"display_url\":\"pic.twitter.com\\/rvyVPAJyxU\",\"expanded_url\":\"https:\\/\\/twitter.com\\/80Level\\/status\\/1329068955009368066\\/video\\/1\",\"type\":\"video\",\"sizes\":{\"thumb\":{\"w\":150,\"h\":150,\"resize\":\"crop\"},\"medium\":{\"w\":1200,\"h\":675,\"resize\":\"fit\"},\"small\":{\"w\":680,\"h\":383,\"resize\":\"fit\"},\"large\":{\"w\":1280,\"h\":720,\"resize\":\"fit\"}},\"video_info\":{\"aspect_ratio\":[16,9],\"duration_millis\":59901,\"variants\":[{\"bitrate\":2176000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/1280x720\\/gAZZ69jdicqnb1st.mp4?tag=10\"},{\"bitrate\":832000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/640x360\\/Uz3JoWHmV6fXlW4f.mp4?tag=10\"},{\"content_type\":\"application\\/x-mpegURL\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/pl\\/KFgpCXp_fwgFZBwX.m3u8?tag=10\"},{\"bitrate\":256000,\"content_type\":\"video\\/mp4\",\"url\":\"https:\\/\\/video.twimg.com\\/ext_tw_video\\/1329049728609378304\\/pu\\/vid\\/480x270\\/0YtecJ3IvviHNOMo.mp4?tag=10\"}]},\"additional_media_info\":{\"monetizable\":false}}]},\"metadata\":{\"iso_language_code\":\"en\",\"result_type\":\"recent\"},\"source\":\"<a href=\\\"https:\\/\\/mobile.twitter.com\\\" rel=\\\"nofollow\\\">Twitter Web App<\\/a>\",\"in_reply_to_status_id\":null,\"in_reply_to_status_id_str\":null,\"in_reply_to_user_id\":null,\"in_reply_to_user_id_str\":null,\"in_reply_to_screen_name\":null,\"user\":{\"id\":2913699628,\"id_str\":\"2913699628\",\"name\":\"80 LEVEL\",\"screen_name\":\"80Level\",\"location\":\"Los Angeles\",\"description\":\"https:\\/\\/t.co\\/TxSqrhGe2F is the best place for game developers, digital artists, animators, video game enthusiasts, CGI and VFX talents.\",\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"entities\":{\"url\":{\"urls\":[{\"url\":\"http:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,22]}]},\"description\":{\"urls\":[{\"url\":\"https:\\/\\/t.co\\/TxSqrhGe2F\",\"expanded_url\":\"http:\\/\\/80.lv\",\"display_url\":\"80.lv\",\"indices\":[0,23]}]}},\"protected\":false,\"followers_count\":65573,\"friends_count\":1586,\"listed_count\":775,\"created_at\":\"Wed Dec 10 12:06:32 +0000 2014\",\"favourites_count\":5780,\"utc_offset\":null,\"time_zone\":null,\"geo_enabled\":true,\"verified\":true,\"statuses_count\":8757,\"lang\":null,\"contributors_enabled\":false,\"is_translator\":false,\"is_translation_enabled\":false,\"profile_background_color\":\"000000\",\"profile_background_image_url\":\"http:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_image_url_https\":\"https:\\/\\/abs.twimg.com\\/images\\/themes\\/theme1\\/bg.png\",\"profile_background_tile\":false,\"profile_image_url\":\"http:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_image_url_https\":\"https:\\/\\/pbs.twimg.com\\/profile_images\\/1179033238792093698\\/7iZDiQf2_normal.jpg\",\"profile_banner_url\":\"https:\\/\\/pbs.twimg.com\\/profile_banners\\/2913699628\\/1569917503\",\"profile_link_color\":\"94D487\",\"profile_sidebar_border_color\":\"000000\",\"profile_sidebar_fill_color\":\"000000\",\"profile_text_color\":\"000000\",\"profile_use_background_image\":false,\"has_extended_profile\":false,\"default_profile\":false,\"default_profile_image\":false,\"following\":false,\"follow_request_sent\":false,\"notifications\":false,\"translator_type\":\"none\"},\"geo\":null,\"coordinates\":null,\"place\":null,\"contributors\":null,\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":90,\"favorited\":false,\"retweeted\":false,\"possibly_sensitive\":false,\"lang\":\"en\"},\"is_quote_status\":false,\"retweet_count\":18,\"favorite_count\":0,\"favorited\":false,\"retweeted\":false,\"lang\":\"en\"}";

        statuses.add(TwitterObjectFactory.createStatus(jsonTweet1));
        statuses.add(TwitterObjectFactory.createStatus(jsonTweet2));

        doReturn(twitterSentimentAnalysis).when(poligraph).getTwitterSentimentAnalysis();
        doReturn(1).when(twitterSentimentAnalysis).analyzeTweet(any(String.class));
        ArrayList<Result> result = poligraph.analyzeSentiment(statuses);
        assertNotNull(result);
        assertThat(result.size(), is(2));
    }
}
